#include "StdAfx.h"
#include "BSLAHack.h"
#include "Config.h"
#include "Utils.h"


static bool Write(HANDLE hFile, LPVOID pData, DWORD dwDataSize)
{
	bool ok = false;
	DWORD dwBytesWrote = 0;
	if(::WriteFile(hFile, pData, dwDataSize, &dwBytesWrote, NULL))
		ok = (dwDataSize == dwBytesWrote);
	return ok;
}

//********************************************************

CConfigPin::CConfigPin(LPCSTR psBS2Name) :
	sBS2Name(psBS2Name)
{
}
CConfigPin::CConfigPin(LPCSTR psBS2Name, PINUSERINFO& pui) :
	sBS2Name(psBS2Name)
{
	Load(pui);
}
CConfigPin::~CConfigPin()
{
}

void CConfigPin::Save(PINUSERINFO& pui)
{
	memcpy(&pui, &userinfo, sizeof(PINUSERINFO));
}

void CConfigPin::Load(PINUSERINFO& pui)
{
	memcpy(&userinfo, &pui, sizeof(PINUSERINFO));
}

void CConfigPin::SetUserText(LPCSTR psUserText)
{
	lstrcpyn(userinfo.szUserText, psUserText, sizeof(userinfo.szUserText)-1);
}

//********************************************************

CConfigProject::CConfigProject(LPCSTR psPrjname) :
	m_sPrjname(psPrjname),
	m_fDirty(false)
{
	for(int p=0; p<NUM_PINS; p++)
		pins[p] = new CConfigPin(theApp.PinNames[p]);

}
#define LOADPIN(zz)	pins[pinidx++] = new CConfigPin(#zz, hdr.##zz)
CConfigProject::CConfigProject(CFG_PRJ_HDR& hdr, PCBYTE pbBmp) :
	m_fDirty(false)
{
	m_sPrjname = hdr.szPrjName;
	int pinidx=0;
	LOADPIN(Sin);
	LOADPIN(Sout);
	LOADPIN(RES);
	LOADPIN(Vdd);
	LOADPIN(P15);
	LOADPIN(P14);
	LOADPIN(P13);
	LOADPIN(P12);
	LOADPIN(P11);
	LOADPIN(P10);
	LOADPIN(P9);
	LOADPIN(P8);
	LOADPIN(P7);
	LOADPIN(P6);
	LOADPIN(P5);
	LOADPIN(P4);
	LOADPIN(P3);
	LOADPIN(P2);
	LOADPIN(P1);
	LOADPIN(P0);
	m_BitmapMem.CopyFrom(pbBmp, hdr.dwBmpSize);
}
CConfigProject::~CConfigProject()
{
	for(int p=0; p<NUM_PINS; p++)
		delete pins[p];
	FreeMem();
}

void CConfigProject::FreeMem()
{
	m_BitmapMem.Free();
}

#define SAVEPIN(zz)	FindPin(#zz)->Save(prjhdr.##zz);
bool CConfigProject::Save(HANDLE hFile)
{
	bool saved = false;

	CFG_PRJ_HDR prjhdr;
	ZeroMemory(&prjhdr, sizeof(CFG_PRJ_HDR));
	lstrcpyn(prjhdr.szPrjName, m_sPrjname, sizeof(prjhdr.szPrjName)-1);
	SAVEPIN(Sin);
	SAVEPIN(Sout);
	SAVEPIN(RES);
	SAVEPIN(Vdd);
	SAVEPIN(P15);
	SAVEPIN(P14);
	SAVEPIN(P13);
	SAVEPIN(P12);
	SAVEPIN(P11);
	SAVEPIN(P10);
	SAVEPIN(P9);
	SAVEPIN(P8);
	SAVEPIN(P7);
	SAVEPIN(P6);
	SAVEPIN(P5);
	SAVEPIN(P4);
	SAVEPIN(P3);
	SAVEPIN(P2);
	SAVEPIN(P1);
	SAVEPIN(P0);
	prjhdr.dwBmpSize = m_BitmapMem.Size();

	if(Write(hFile, &prjhdr, sizeof(prjhdr)))
		saved = Write(hFile, (LPVOID)m_BitmapMem.Data(), m_BitmapMem.Size());

	m_fDirty = !saved;
	return saved;
}

void CConfigProject::SetPrjName(LPCSTR psPrjname)
{
	if(m_sPrjname != psPrjname)
	{
		m_sPrjname = psPrjname;
		m_fDirty = true;
	}
}

CString CConfigProject::GetPinUserText(LPCSTR psBS2Name)
{
	CString sTxt;
	CConfigPin* pPin = FindPin(psBS2Name);
	if(pPin)
		sTxt = pPin->userinfo.szUserText;
	return sTxt;
}

void CConfigProject::SetPinUserText(LPCSTR psBS2Name, LPCSTR psUserTxt)
{
	CConfigPin* pPin = FindPin(psBS2Name);
	if(pPin)
	{
		if(pPin->userinfo.szUserText != psUserTxt)
		{
			pPin->SetUserText(psUserTxt);
			m_fDirty = true;
		}
	}
}

COLORREF CConfigProject::GetPinColor(LPCSTR psPinName, bool fText)
{
	CConfigPin* pPin = FindPin(psPinName);
	if(!pPin)
		return NO_COLOR;

	return fText ? pPin->userinfo.crTxt : pPin->userinfo.crBg;
}

void CConfigProject::SetPinColor(LPCSTR psPinName, COLORREF rgb, bool fText)
{
	CConfigPin* pPin = FindPin(psPinName);
	if(pPin)
	{
		COLORREF& cr = fText? pPin->userinfo.crTxt : pPin->userinfo.crBg;
		cr = rgb;
		m_fDirty = true;
	}
}

CConfigPin* CConfigProject::FindPin(LPCSTR psBS2Name)
{
	for(int p=0; p<NUM_PINS; p++)
	{
		if(!pins[p]->sBS2Name.CompareNoCase(psBS2Name))
			return pins[p];
	}
	return NULL;
}

bool CConfigProject::SetBitmap(CMemBuf& buf)
{
	FreeMem();
	m_BitmapMem.CopyFrom(buf);
	m_fDirty = true;
	return m_BitmapMem.HasData();
}

//********************************************************

CConfig::CConfig(void) :
	m_fDirty(false)
{
	char szFile[1024] = {0};
	::GetModuleFileName(NULL, szFile, sizeof(szFile));
	char* ext = strrchr(szFile, '.');
	lstrcpy(ext, ".hak");
	m_sConfigFile = szFile;
}

CConfig::~CConfig(void)
{
	Clear();
}

void CConfig::Clear()
{
	for(int idx=0; idx<m_projects.GetCount(); idx++)
		delete m_projects.GetAt(idx);
	m_projects.RemoveAll();
}

bool CConfig::Load()
{
	m_fDirty = false;

	Clear();

	CString msg("Load from ");
	msg += m_sConfigFile;
	AfxMessageBox(msg);


	CMemBuf buf;
	if(buf.LoadFromFile(m_sConfigFile))
	{
		DWORD sz = buf.Size();
		PCBYTE pb = buf.Data();
		if(sz > sizeof(CFG_FILE_HDR))
		{
			CFG_FILE_HDR* pHdr = (CFG_FILE_HDR*)pb;
			if(!memcmp(pHdr->szTag, HDR_TAG, sizeof(pHdr->szTag)))
			{
				m_sCurrPrjName = pHdr->szCurrPrjName;
				pb += sizeof(CFG_FILE_HDR);
				sz -= sizeof(CFG_FILE_HDR);
				while(sz > 0)
				{
					CFG_PRJ_HDR* pPrjHdr = (CFG_PRJ_HDR*)pb;
					pb += sizeof(CFG_PRJ_HDR);
					sz -= sizeof(CFG_PRJ_HDR);
					if(sz >= pPrjHdr->dwBmpSize)
					{
						CConfigProject* pPrj = new CConfigProject(*pPrjHdr, pb);
						m_projects.Add(pPrj);
						pb += pPrjHdr->dwBmpSize;
						sz -= pPrjHdr->dwBmpSize;
					}
					else
					{
						break;
					}
				}
			}
		}
	}
	else
	{
	}

	return (m_projects.GetSize() > 0);
}

bool CConfig::Save()
{
	bool saved = false;

	CString msg("Save to ");
	msg += m_sConfigFile;
	AfxMessageBox(msg);

	HANDLE hFile = CreateFile(	m_sConfigFile,
								GENERIC_WRITE,
								0,
								NULL,
								CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL,
								NULL);

	if(INVALID_HANDLE_VALUE != hFile)
	{
		CFG_FILE_HDR hdr;
		memcpy(hdr.szTag, HDR_TAG, sizeof(hdr.szTag));
		hdr.dwVer = 1;
		lstrcpyn(hdr.szCurrPrjName, m_sCurrPrjName, sizeof(hdr.szCurrPrjName)-1);

		if(Write(hFile, &hdr, sizeof(hdr)))
		{
			saved = true;
			for(int idx=0; idx<m_projects.GetCount(); idx++)
				saved &= m_projects.GetAt(idx)->Save(hFile);
		}

		CloseHandle(hFile);
	}
	m_fDirty = !saved;
	return saved;
}

bool CConfig::IsDirty()
{
	if(m_fDirty)
		return true;
	for(int idx=0; idx<m_projects.GetCount(); idx++)
	{
		if(m_projects.GetAt(idx)->IsDirty())
			return true;
	}
	return false;
}

CString CConfig::GetCurrentProjectName()
{
	return m_sCurrPrjName;
}

void CConfig::SetCurrentProjectName(LPCSTR psPrjname)
{
	m_sCurrPrjName = psPrjname;
	m_fDirty = true;
}

CConfigProject* CConfig::GetProject(LPCSTR psPrjname)
{
	for(int idx=0; idx<m_projects.GetCount(); idx++)
	{
		CConfigProject* prj = m_projects.GetAt(idx);
		if(prj->GetPrjName() == psPrjname)
			return prj;
	}
	return NULL;
}

CConfigProject* CConfig::GetProject(int idx)
{
	if( (idx>=0) && (idx<GetProjectCount()) )
		return m_projects.GetAt(idx);
	return NULL;
}

CConfigProject* CConfig::GetCurrentProject()
{
	return GetProject(m_sCurrPrjName);
}

CConfigProject* CConfig::CreateProject(LPCSTR psPrjname)
{
	CConfigProject* prj = new CConfigProject(psPrjname);
	m_projects.Add(prj);
	m_fDirty = true;
	return prj;
}

bool CConfig::GetCurrentProjectPinInfo(LPCSTR psBS2Name, PINUSERINFO& pui)
{
	CConfigProject* pPrj = GetCurrentProject();
	if(!pPrj)
		return false;

	CConfigPin* pPin = pPrj->FindPin(psBS2Name);
	if(!pPin)
		return false;

	memcpy(&pui, &pPin->userinfo, sizeof(PINUSERINFO));
	return true;
}
