#pragma once

#define NUM_PINS	20
#define NO_COLOR	((COLORREF)(-1))

#include "MemBuf.h"

//************************************

struct PINUSERINFO
{
	char szUserText[8];
	COLORREF crBg;
	COLORREF crTxt;
	PINUSERINFO() :
		crBg(NO_COLOR),
		crTxt(NO_COLOR)
	{
		ZeroMemory(&szUserText, sizeof(szUserText));
	}
};

#define HDR_TAG	"CFH\0"
struct CFG_FILE_HDR
{
	char szTag[4];	// == HDR_TAG
	DWORD dwVer;
	char szCurrPrjName[32];
};
struct CFG_PRJ_HDR
{
	char szPrjName[32];
	PINUSERINFO	Sin;
	PINUSERINFO	Sout;
	PINUSERINFO	RES;
	PINUSERINFO	Vdd;
	PINUSERINFO	P15;
	PINUSERINFO	P14;
	PINUSERINFO	P13;
	PINUSERINFO	P12;
	PINUSERINFO	P11;
	PINUSERINFO	P10;
	PINUSERINFO	P9;
	PINUSERINFO	P8;
	PINUSERINFO	P7;
	PINUSERINFO	P6;
	PINUSERINFO	P5;
	PINUSERINFO	P4;
	PINUSERINFO	P3;
	PINUSERINFO	P2;
	PINUSERINFO	P1;
	PINUSERINFO	P0;
	DWORD dwBmpSize;
	// bitmap file data follows
};

//************************************
class CConfigPin
{
public:
	CConfigPin(LPCSTR psName);
	CConfigPin(LPCSTR psName, PINUSERINFO& pui);
	~CConfigPin();

	void Save(PINUSERINFO& pui);
	void Load(PINUSERINFO& pui);
	void SetUserText(LPCSTR psUserText);

	CString sBS2Name;
	PINUSERINFO userinfo;
};

//************************************
struct CFG_PRJ_HDR;
class CConfigProject
{
friend class CConfig;
public:
	CConfigProject(LPCSTR psPrjname);
	CConfigProject(CFG_PRJ_HDR& hdr, PCBYTE pbBmp);
	~CConfigProject();

	CString GetPrjName()				{return m_sPrjname;}
	void SetPrjName(LPCSTR psPrjname);

	CConfigPin* FindPin(LPCSTR psBS2Name);

	CString GetPinUserText(LPCSTR psBS2Name);
	void SetPinUserText(LPCSTR psBS2Name, LPCSTR psTxt);

	COLORREF GetPinColor(LPCSTR psPinName, bool fText);
	void SetPinColor(LPCSTR psPinName, COLORREF rgb, bool fText);

	bool SetBitmap(CMemBuf& buf);
	CMemBuf& GetBitmapBuf()		{return m_BitmapMem;}

	bool IsDirty()	{return m_fDirty;}

protected:
	CString m_sPrjname;
	bool m_fDirty;
	CConfigPin* pins[NUM_PINS];
	CMemBuf m_BitmapMem;

	bool Save(HANDLE hFile);
	void FreeMem();
};

//************************************
class CConfig
{
public:
	CConfig();
	~CConfig();

	bool Load();
	bool Save();

	CString GetCurrentProjectName();
	void SetCurrentProjectName(LPCSTR psPrjname);

	CConfigProject* GetProject(LPCSTR psPrjname);
	CConfigProject* GetProject(int idx);
	CConfigProject* GetCurrentProject();

	int GetProjectCount()	{return m_projects.GetSize();}
	CConfigProject* CreateProject(LPCSTR psPrjname);
	bool IsDirty();
	bool GetCurrentProjectPinInfo(LPCSTR psBS2Name, PINUSERINFO& pui);

protected:
	CString m_sConfigFile;
	CString m_sCurrPrjName;
	bool m_fDirty;
	CTypedPtrArray<CPtrArray, CConfigProject*> m_projects;

	void Clear();
};
