// MainWnd.cpp : implementation file
//

#include "stdafx.h"
#include "BSLAHack.h"
#include "HackWnd.h"
#include "HackDefs.h"
#include "FindBSLA.h"
#include "Utils.h"

// CHackWnd

IMPLEMENT_DYNAMIC(CHackWnd, CWnd)

CHackWnd::CHackWnd()
{
}

CHackWnd::~CHackWnd()
{
	FreeMem();
}

void CHackWnd::FreeMem()
{
	m_MemBuf.Free();
}

BOOL CHackWnd::Create(HWND hParent)
{
    WNDCLASS wc;
    memset(&wc, 0, sizeof(WNDCLASS));
    wc.style = NULL;
    wc.lpfnWndProc = ::DefWindowProc;
    wc.hInstance = AfxGetInstanceHandle();
    wc.hIcon = theApp.LoadIcon(IDR_MAINFRAME);
    wc.hCursor = theApp.LoadStandardCursor(IDC_ARROW);
    wc.hbrBackground = (HBRUSH)::GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = HACK_CONTROL_WNDCLASS;
    if (!AfxRegisterClass(&wc))
    {
        TRACE0("Class Registration Failed\n");
        return FALSE;
    }

	if(!CWnd::CreateEx(0, HACK_CONTROL_WNDCLASS, HACK_CONTROL_WNDTITLE, WS_CHILD|WS_VISIBLE, 24, 31, HACKWND_CX, HACKWND_CY, hParent, 0))
		return FALSE;

	UpdateWindow();
//	ShowWindow(SW_SHOW);
	return TRUE;
}

CPinCtrl* CHackWnd::FindPin(LPCSTR psPinName)
{
	for(int p=0; p<NUM_PINS; p++)
	{
		if(!m_pins[p].m_sBS2Name.CompareNoCase(psPinName))
			return &m_pins[p];
	}
	return NULL;
}

void CHackWnd::SetPinUserText(LPCSTR psBS2Name, LPCSTR psUserTxt)
{
	CPinCtrl* pPin = FindPin(psBS2Name);
	if(pPin)
	{
		pPin->SetUserText(psUserTxt);
		pPin->Invalidate();
	}
}

CString CHackWnd::GetPinUserText(LPCSTR psBS2Name)
{
	CPinCtrl* pPin = FindPin(psBS2Name);
	return pPin ? pPin->GetUserText() : "";
}

void CHackWnd::SetPinColor(LPCSTR psPinName, COLORREF rgb, bool fText)
{
	CPinCtrl* pPin = FindPin(psPinName);
	if(pPin)
	{
		COLORREF& cr = fText ? pPin->m_PinUserInfo.crTxt : pPin->m_PinUserInfo.crBg;
		cr = rgb;
		pPin->Invalidate();
	}
}

COLORREF CHackWnd::GetPinColor(LPCSTR psPinName, bool fText)
{
	CPinCtrl* pPin = FindPin(psPinName);
	if(!pPin)
		return NO_COLOR;
	return fText ? pPin->m_PinUserInfo.crTxt : pPin->m_PinUserInfo.crBg;
}

void CHackWnd::SetBitmap(LPCSTR psBmpPath)
{
	CMemBuf tmp;
	tmp.LoadFromFile(psBmpPath);
	SetBitmap(tmp);
}

void CHackWnd::SetBitmap(CMemBuf& buf)
{
	FreeMem();
	m_MemBuf.CopyFrom(buf);
	m_bmp.LoadFromMem(m_MemBuf);
	Invalidate();
}

static BOOL CALLBACK PinEnumProc(HWND hChild, LPARAM)
{
	::PostMessage(hChild, WMAPP_RELOAD, 0, 0);
	return TRUE;
}

void CHackWnd::Reload()
{
	CConfigProject* pPrj = theApp.Config.GetCurrentProject();
	if(pPrj)
		this->m_bmp.LoadFromMem(pPrj->GetBitmapBuf());

	::EnumChildWindows(m_hWnd, PinEnumProc, 0);

	Invalidate();
}

BEGIN_MESSAGE_MAP(CHackWnd, CWnd)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_MESSAGE(WMAPP_RELOAD, OnReload)
END_MESSAGE_MAP()

// CHackWnd message handlers

int CHackWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
//	m_bmp.Load();

	int x = HACKWND_CX - (CX_PINWND+2);
	int y = 2;

	UINT uIdc = PINWND_ID_BASE;
	for(int p=0; p<PIN_COUNT; p++)
	{
		m_pins[p].Create(this, x, y, theApp.PinNames[p], uIdc);
		y += CY_PINWND + 2;
		uIdc++;
	}

	Reload();

	return 0;
}

void CHackWnd::OnPaint() 
{
	CPaintDC dcPaint(this); // device context for painting
	if(!m_bmp.Draw(dcPaint))
	{
		CRect rc;
		GetClientRect(&rc);
		rc.left = (rc.Width()/2) - 50;
		rc.right = rc.left + 80;
		rc.top = (rc.Height()/2) - 10;
		rc.bottom = rc.top + 20;
		dcPaint.DrawText("(no image)", &rc, DT_LEFT|DT_VCENTER);
	}
}

LRESULT CHackWnd::OnReload(WPARAM, LPARAM)
{
	Reload();
	return 0;
}
