#include "StdAfx.h"
#include "MemBuf.h"

CMemBuf::CMemBuf(void) :
	m_pb(NULL),
	m_sz(0)
{
}

CMemBuf::~CMemBuf(void)
{
	Free();
}

void CMemBuf::Free()
{
	if(m_pb)
		delete [] m_pb;
	m_pb = NULL;
	m_sz = 0;
}

bool CMemBuf::Alloc(DWORD sz)
{
	Free();
	m_pb = new BYTE[sz];
	if(!m_pb)
		return false;
	m_sz = sz;
	return HasData();
}

bool CMemBuf::HasData()
{
	return ( (m_sz!=0) && (m_pb!=NULL) );
}

bool CMemBuf::LoadFromFile(LPCSTR psFile)
{
	Free();

	HANDLE hFile = CreateFile(	psFile,
								GENERIC_READ,
								FILE_SHARE_READ,
								NULL,
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL);
	if(INVALID_HANDLE_VALUE != hFile)
	{
		DWORD dwSize = GetFileSize(hFile, NULL);
		if(Alloc(dwSize))
		{
			bool ok = false;
			DWORD nr = 0;
			if(ReadFile(hFile, m_pb, m_sz, &nr, NULL))
				ok = (m_sz == nr);
			if(!ok)
				Free();
		}
		CloseHandle(hFile);
	}
	return HasData();
}

bool CMemBuf::CopyFrom(CMemBuf& src)
{
	return CopyFrom(src.m_pb, src.m_sz);
}

bool CMemBuf::CopyFrom(PCBYTE pb, DWORD sz)
{
	Free();
	if(pb && (sz>0))
	{
		if(Alloc(sz))
			memcpy(m_pb, pb, sz);
	}
	return HasData();
}


HGLOBAL CMemBuf::ToHGlobal()
{
	HGLOBAL hRtn = NULL;
	if(HasData())
	{
		HGLOBAL hTmp = GlobalAlloc(GMEM_MOVEABLE, m_sz);
		if(hTmp)
		{
			LPVOID pvData = GlobalLock(hTmp);
			if(pvData)
			{
				memcpy(pvData, m_pb, m_sz);
				hRtn = hTmp;
			}
			::GlobalUnlock(hTmp);
			if(hRtn == NULL)
				GlobalFree(hTmp);	
		}
	}
	return hRtn;
}
