// Reg.cpp: implementation of the CReg class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Reg.h"

//#include <atlbase.h>	// needed for CReg::RecurseDeleteKey().  GFC version strrchr and is more code.

#define s_sec NULL

//////////////////////////////////////////////////////////////////////
//
// This class is a thin wrapper around the Win32 APIs.
// It can be used with ATL/WTL without GString conflicts.
//
//////////////////////////////////////////////////////////////////////

#define CHKREG(z)		((z) == ERROR_SUCCESS)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CReg::CReg()
{

}

CReg::~CReg()
{

}




BOOL CReg::WriteDWORD(HKEY branch, LPCSTR keyName, LPCSTR valueName, DWORD value)
{
	BOOL ret = FALSE;	// pessimist

	// Check the parameters
	if(branch && keyName && *keyName)
	{
		// Okay

		// Open the key
		HKEY hkey;
		DWORD disposition;
		if(CHKREG(RegCreateKeyEx(branch, keyName, 0, NULL, 0, KEY_WRITE, s_sec, &hkey, &disposition)))
		{
			// Okay

			// Set the value
			if(CHKREG(RegSetValueEx(hkey, valueName, 0, REG_DWORD, (LPBYTE)&value, sizeof(DWORD))))
			{
				ret = TRUE;
			}
			else
			{
				//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryWriteDWORD no set %s", SafeStr(valueName));
			}

			// Close the key
			RegCloseKey(hkey);
		}
		else
		{
			//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryWriteDWORD no open/create %s", SafeStr(keyName));
		}
	}
	else
	{
		//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryWriteDWORD params");
	}

	// Return result
	return ret;
}


BOOL CReg::ReadDWORD(HKEY branch, LPCSTR keyName, LPCSTR valueName, DWORD& value, DWORD defaultValue /*=0*/)
{
	BOOL ret = FALSE;	// pessimist

	// Clear the out DWORD
	value = defaultValue;

	// Check the parameters
	if(branch && keyName && *keyName)
	{
		// Okay

		// Open the key
		HKEY hkey;
		if(CHKREG(RegOpenKeyEx(branch, keyName, 0, KEY_READ, &hkey)))
		{
			// Opened

			DWORD length=sizeof(DWORD);
			DWORD type=0;

			// Query the value
			if(CHKREG(RegQueryValueEx(hkey, valueName, 0, &type, (LPBYTE)&value, &length)))
			{
				// Is it a DWORD?
				if (type == REG_DWORD)
				{
					// Success
					ret = TRUE;
				}
				else
				{
					//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryReadDWORD %s not REG_DWORD", SafeStr(valueName));
				}
			}
			else
			{
				//eGuardError::LogFormat(AsMessage, __FILE__, __LINE__, "RegistryReadDWORD no query %s", SafeStr(valueName));
			}

			// Close the key
			RegCloseKey(hkey);
		}
		else
		{
			//eGuardError::LogFormat(AsMessage, __FILE__, __LINE__, "RegistryReadDWORD no open %s", SafeStr(keyName));
		}
	}
	else
	{
		//eGuardError::Log(AsError, __FILE__, __LINE__, "RegistryReadDWORD params");
	}

	// Return result
	return ret;
}



BOOL CReg::WriteString(HKEY branch, LPCSTR keyName, LPCSTR valueName, LPCSTR strValue)
{
	BOOL ret = FALSE;	// pessimist

	// Check the parameters
	if(branch && keyName && *keyName && strValue)
	{
		// Okay

		// Open the key
		HKEY hkey;
		DWORD disposition;
		if(CHKREG(RegCreateKeyEx(branch, keyName, 0, NULL, 0, KEY_WRITE, s_sec, &hkey, &disposition)))
		{
			// Okay

			// Set the value
			if(CHKREG(RegSetValueEx(hkey, valueName, 0, REG_SZ, (LPBYTE)strValue, lstrlen(strValue)+1)))
			{
				ret = TRUE;
			}
			else
			{
				//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryWriteString no set %s", SafeStr(valueName));
			}

			// Close the key
			RegCloseKey(hkey);
		}
		else
		{
			//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryWriteString no open/create %s", SafeStr(keyName));
		}
	}
	else
	{
		//eGuardError::Log(AsError, __FILE__, __LINE__, "RegistryWriteString params");
	}

	// Return result
	return ret;
}


//
// Caller must delete "value" that we return.
//
CString CReg::ReadString(HKEY branch, LPCSTR keyName, LPCSTR valueName)
{
	CString ret;

	if(branch && keyName && *keyName)	// OK for valueName to be NULL, that means default key
	{
		HKEY hkey;
		if(CHKREG(RegOpenKeyEx(branch, keyName, 0, KEY_READ, &hkey)))
		{
			DWORD length=0;
			DWORD type=0;
			if(CHKREG(RegQueryValueEx(hkey, valueName, 0, &type, NULL, &length)))
			{
				if ( (type == REG_SZ) || (type==REG_EXPAND_SZ) )
				{
					char value[1024];
					DWORD dwSz = sizeof(value);
					if(CHKREG(RegQueryValueEx(hkey, valueName, 0, &type, (BYTE*)value, &dwSz)))
					{
						if (type==REG_EXPAND_SZ)
						{
							char expanded[1024];
							::ExpandEnvironmentStrings(value, expanded, sizeof(expanded));
							ret = expanded;
						}
						else
						{
							ret = value;
						}
					}
				}
			}
			RegCloseKey(hkey);
		}
	}

	return ret;
}



BOOL CReg::CreateKey(HKEY branch, LPCSTR keyName)
{
	BOOL ret = FALSE;	// pessimist

	// Check the parameters
	if(branch && keyName && *keyName)
	{
		// Okay

		// Create the key
		HKEY hkey;
		DWORD disposition;
		if(CHKREG(RegCreateKeyEx(branch, keyName, 0, NULL, 0, KEY_WRITE, s_sec, &hkey, &disposition)))
		{
			// Close the key
			RegCloseKey(hkey);
			ret = TRUE;
		}
	}

	return ret;
}

#if 0

BOOL CReg::DeleteKey(HKEY branch, LPCSTR keyName)
{
	// Check the parameters
	if(branch && keyName && *keyName)	// valueName can be NULL/default
	{
		CRegKey regkey;
		LONG rc = regkey.Open(branch, NULL);	// open top level root -- this must be above the key we're deleting, not the same as it.
		if (rc == ERROR_SUCCESS)
		{
			rc = regkey.RecurseDeleteKey(keyName);
			regkey.Close();
			if (rc == ERROR_SUCCESS)
			{
				return TRUE;
			}
		}
	}
	return FALSE;
}


BOOL CReg::DeleteKey(HKEY branch, LPCSTR keyName)
{
	BOOL fOK = FALSE;
	char szRoot[1024];
	char* psSubkey;

	if(branch && keyName && *keyName)
	{
		strcpy(szRoot, keyName);
		psSubkey = strrchr(szRoot, '\\');
		if(!psSubkey)
			return FALSE;

		*psSubkey = 0;
		psSubkey++;

		HKEY hkRoot=NULL;
		if(CHKREG(RegOpenKeyEx(branch, szRoot, 0, KEY_READ|KEY_WRITE, &hkRoot)))
		{
			// RegDeleteKey() will return an error if the key doesn't exist,
			// so we need to verify it's existance first.
			BOOL fFound = FALSE;
			DWORD dwIndex = 0;
			char szName[MAX_PATH+1];
			while (!fFound && (RegEnumKey(hkRoot,dwIndex++,szName, sizeof(szName)) == ERROR_SUCCESS) )
			{
				fFound = ! lstrcmpi(szName, psSubkey);
			}

			if(fFound)
			{
				fOK = TRUE;
				// On NT, we need to delete each sub key ourselves.
				char szTemp[1024];
				dwIndex = 0;	// start back at beginning of key -- ALWAYS.  
								// Don't iterate beyond 0, since we're deleting keys.

				// Open the child key, and iterate through it
				HKEY hkCurrent = NULL;
				if (RegOpenKeyEx(branch, keyName, 0, KEY_READ|KEY_WRITE, &hkCurrent) == ERROR_SUCCESS)
				{
					DWORD err2 = 0;
					while (fOK && ((err2 = RegEnumKey(hkCurrent, dwIndex, szName, sizeof(szName))) == ERROR_SUCCESS) )
					{
						// Call ourselves recursively, to delete all child keys
						lstrcpy(szTemp, keyName);
						lstrcat(szTemp, "\\");
						lstrcat(szTemp, szName);
						fOK = DeleteKey(branch, szTemp);
					}
					if (err2 != ERROR_NO_MORE_ITEMS)
					{
						// oh no!
						err2 = err2;
					}
					// All child keys have been deleted.  Close the key
					RegCloseKey(hkCurrent);
				}
				// All child keys have been deleted.  Delete the current key.
				DWORD err = RegDeleteKey(hkRoot, psSubkey); 
				if(!CHKREG(err))
				{
					fOK = FALSE;
					err = err;
					//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryDeleteKey no delete %s\\%s", szRoot, SafeStr(psSubkey));
				}
			}
			else
			{
				fOK = TRUE;	// As good as deleted
			}

			RegCloseKey(hkRoot);
		}
		else
		{
			// DWORD err = GetLastError(); // eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryDeleteKey no open %s", szRoot);
		}
	}
	else
	{
		// DWORD err = GetLastError(); //eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryDeleteKey params");
	}

	return fOK;
}
#endif


BOOL CReg::DeleteValue(HKEY branch, LPCSTR keyName, LPCSTR valueName)
{
	BOOL ret = FALSE;	// pessimist

	// Check the parameters
	if(branch && keyName && *keyName)
	{
		// Okay

		// Open the key
		HKEY hkey;
		if(CHKREG(RegOpenKeyEx(branch, keyName, 0, KEY_WRITE, &hkey)))
		{
			ret = (CHKREG(RegDeleteValue(hkey, valueName)));

			// Close the key
			RegCloseKey(hkey);
		}
		else
		{
			//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryDeleteValue no open %s", SafeStr(keyName));
		}
	}
	else
	{
		//eGuardError::LogFormat(AsError, __FILE__, __LINE__, "RegistryDeleteValue params");
	}

	return ret;
}


BOOL CReg::KeyExists(HKEY branch, LPCSTR keyName)
{
	BOOL exists = FALSE;

	if (branch && keyName && *keyName)
	{
		// Open the key	(could use CRegKey::Open() instead, either way.)
		HKEY hkOpen;
		if (ERROR_SUCCESS == RegOpenKeyEx(branch, keyName, 0, KEY_READ, &hkOpen))
		{
			exists = TRUE;

			// Close the key
			RegCloseKey(hkOpen);
		}
	}

	return exists;
}


BOOL CReg::ValueExists(HKEY branch, LPCSTR keyName, LPCSTR valueName)
{
	BOOL exists = FALSE;

	if (branch && keyName && *keyName)
	{
		// Open the key
		HKEY hkOpen;
		if (ERROR_SUCCESS == RegOpenKeyEx(branch, keyName, 0, KEY_READ, &hkOpen))
		{
			// Query the value's type - REG_NONE indicates not present.
			DWORD dwType=REG_NONE;
			if (ERROR_SUCCESS == RegQueryValueEx(hkOpen, valueName, NULL, &dwType, NULL, NULL))
			{
				exists = (REG_NONE != dwType);
			}

			// Close the key
			RegCloseKey(hkOpen);
		}
	}

	return exists;
}


void CReg::Test()
{
#if 0 // def _DEBUG
	CReg::WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test", "Dummy", 123);
	if (CReg::ValueExists(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test", "Dummy"))
	{
		CReg::DeleteValue(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test", "Dummy");
		CReg::WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2", "Dummy2", 456);
		CReg::WriteString(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2", "DummyStr", "Hello CReg");
		LPTSTR strRet;
		CReg::ReadString(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2", "DummyStr", strRet);
		delete[] strRet;
		CReg::WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2\\Level3\\Level4", "Dummy4", 678);
		CReg::WriteDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2\\Level3", "Dummy3", 789);
		DWORD val;
		CReg::ReadDWORD(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2\\Level3", "Dummy3", val);
		if (CReg::KeyExists(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test\\Level2\\Level3"))
		{
			CReg::DeleteKey(HKEY_LOCAL_MACHINE, "SOFTWARE\\Gator.com\\CReg-Test");
		}
	}
#endif
}
