// Author: Dave Chambers (dlchambers@aol.com)
// BSLAHack project webpage: http://www.dlwrr.com/electronics/tools/bsla/hack/
#include "stdafx.h"
#include "TrayIconMgr.h"
#include "Utils.h"

#define MY_ICON_ID	1	// tray icon# that this obj manages

/////////////////////////////////////////////////////////////////////////////
// CTrayIconMgr

CTrayIconMgr::CTrayIconMgr() :
	m_hWnd(NULL),
	m_uCallbackMsg(0),
	m_hIcon(NULL),
	m_uIdIcon(-1),
	m_uTaskbarCreatedMsg(0),
	m_hTrayNotifyWnd(NULL),
	//m_psTip(NULL),
	m_fInTray(FALSE)
{
	// Retrieve the ID of the taskbar restart message
	m_uTaskbarCreatedMsg = RegisterWindowMessage(TEXT("TaskbarCreated"));

	// Get the tray notify window handle, for animation on open/close
	HWND hTray = FindShellTrayWnd();
	if (hTray)
		m_hTrayNotifyWnd = ::FindWindowEx(hTray, NULL, "TrayNotifyWnd", NULL);

	// Don't call NIM_ADD until Create()
}

CTrayIconMgr::~CTrayIconMgr()
{
	ASSERT(!m_fInTray);	// If this asserts, you forgot to call Destroy() in your OnDestroy()
	_setTipText(NULL);
}

void CTrayIconMgr::Create(HWND hWnd, UINT uCallbackMsg, UINT uIdIcon, LPCSTR psTip)
{
	m_hWnd = hWnd;
	ASSERT(IsWindow(m_hWnd));	// Wnd must have already been created!
	m_uCallbackMsg = uCallbackMsg;

	_loadIcon(uIdIcon);

	_setTipText(psTip);

	TrayMessage(NIM_ADD);
}

void CTrayIconMgr::_loadIcon(UINT uIdIcon)
{
	if(uIdIcon != m_uIdIcon)
	{
		m_hIcon = (HICON)::LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(uIdIcon), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		m_uIdIcon = uIdIcon;
	}
}

void CTrayIconMgr::SetIcon(UINT uIdIcon)
{
	//m_sTip.Empty();
	_loadIcon(uIdIcon);
	TrayMessage(NIM_MODIFY);
}

void CTrayIconMgr::_setTipText(LPCSTR psTip)
{
	bool same = (*psTip && (psTip == m_sTip));
	if(!same)
	{
		m_sTip.Empty();
		if(psTip)
		{
			m_sTip = psTip;
			HWND hTray = FindShellTrayWnd();
			::SetWindowPos(hTray, HWND_TOPMOST, 0,0,0,0, SWP_NOMOVE|SWP_NOSIZE);

			// Workaround for the tooltip-behind-taskbar bug in XP:
			//::EnumWindows(CTrayIconMgr::_enumTooltipCallback, 0);
		}
	}
}

void CTrayIconMgr::SetTipText(LPCSTR psTip)
{
	_setTipText(psTip);
	TrayMessage(NIM_MODIFY);
}

void CTrayIconMgr::TrayMessage(UINT message)
{
   	NOTIFYICONDATA  tnd;
	ZeroMemory(&tnd, sizeof(tnd));

	lstrcpyn(tnd.szTip, m_sTip, sizeof(tnd.szTip));

    switch (message)
    {
        case NIM_ADD:
			// Create our icon in the tray
            tnd.uFlags = NIF_MESSAGE;
			if(m_hIcon)
				tnd.uFlags |= NIF_ICON;
			if(!m_sTip.IsEmpty())
				tnd.uFlags |= NIF_TIP;

			m_fInTray = TRUE;

            break;

        case NIM_MODIFY:
			// Update our icon in the tray
			ASSERT(m_fInTray);	// Need to add it 1st!
			if(m_hIcon)
				tnd.uFlags |= NIF_ICON;
			if(!m_sTip.IsEmpty())
				tnd.uFlags |= NIF_TIP;
            break;

        case NIM_DELETE:
			// Delete our icon from the tray
            tnd.uFlags = 0;
			m_fInTray = FALSE;
            break;
    }

	// Send our info to the shell
    tnd.uID = (UINT)MY_ICON_ID;
    tnd.cbSize		= sizeof(NOTIFYICONDATA);
    tnd.hWnd		= m_hWnd;
    tnd.uCallbackMessage = m_uCallbackMsg;
	tnd.hIcon = m_hIcon;

    Shell_NotifyIcon(message, &tnd);
}

///*static*/ BOOL CALLBACK CTrayIconMgr::_enumTooltipCallback(HWND hWnd, LPARAM lParam)
//{
//	char szClass[256];
//	::GetClassName(hWnd, szClass, sizeof(szClass));
////	if(!lstrcmpi("tooltips_class32", szClass))
//	//if(!lstrcmpi("Shell_TrayWnd", szClass))
//	//	::SetWindowPos(hWnd, HWND_TOPMOST, 0,0,0,0, SWP_NOMOVE|SWP_NOSIZE);
//	return TRUE;
//}

UINT CTrayIconMgr::OnTrayCallback(WPARAM wParam, LPARAM lParam)
{
	UINT uRet = 0;
	if (MY_ICON_ID == wParam)
		uRet = (UINT)lParam;	// This is the mouse message, eg. WM_LBUTTONUP
	return uRet;
}

bool CTrayIconMgr::_getTrayRect(RECT& rc)
{
	// Draw animated rects
	RECT rtTray = {0,0,0,0};
	if (m_hTrayNotifyWnd)
	{
		::GetWindowRect(m_hTrayNotifyWnd, &rc);
		rc.right -= 50;	// ignore clock, assumes taskbar is on bottom!
	}
	
	return ( (rc.bottom>rc.top) && (rc.right>rc.left) );
}

/*static*/ HWND CTrayIconMgr::FindShellTrayWnd()
{
	return ::FindWindow("Shell_TrayWnd", NULL);
}